@@ -29,7 +29,6 @@
 #include "gtkhandlebox.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
 #include "gtkwindow.h"
 #include "gtkintl.h"
 
@@ -136,26 +135,28 @@ static GtkBinClass *parent_class;
 static guint        handle_box_signals[SIGNAL_LAST] = { 0 };
 
 
-GtkType
+GType
 gtk_handle_box_get_type (void)
 {
-  static GtkType handle_box_type = 0;
+  static GType handle_box_type = 0;
 
   if (!handle_box_type)
     {
-      static const GtkTypeInfo handle_box_info =
+      static const GTypeInfo handle_box_info =
       {
-	"GtkHandleBox",
-	sizeof (GtkHandleBox),
 	sizeof (GtkHandleBoxClass),
-	(GtkClassInitFunc) gtk_handle_box_class_init,
-	(GtkObjectInitFunc) gtk_handle_box_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_handle_box_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkHandleBox),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_handle_box_init,
       };
 
-      handle_box_type = gtk_type_unique (GTK_TYPE_BIN, &handle_box_info);
+      handle_box_type = g_type_register_static (GTK_TYPE_BIN, "GtkHandleBox",
+						&handle_box_info, 0);
     }
 
   return handle_box_type;
@@ -174,7 +175,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
   widget_class = (GtkWidgetClass *) class;
   container_class = (GtkContainerClass *) class;
 
-  parent_class = gtk_type_class (GTK_TYPE_BIN);
+  parent_class = g_type_class_peek_parent (class);
 
   gobject_class->set_property = gtk_handle_box_set_property;
   gobject_class->get_property = gtk_handle_box_get_property;
@@ -182,7 +183,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW,
                                    g_param_spec_enum ("shadow", NULL,
-                                                      _("Deprecated property, use shadow_type instead."),
+                                                      _("Deprecated property, use shadow_type instead"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_ETCHED_OUT,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -190,7 +191,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
                                    PROP_SHADOW_TYPE,
                                    g_param_spec_enum ("shadow_type",
                                                       _("Shadow type"),
-                                                      _("Appearance of the shadow that surrounds the container."),
+                                                      _("Appearance of the shadow that surrounds the container"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_ETCHED_OUT,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -199,7 +200,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
                                    PROP_HANDLE_POSITION,
                                    g_param_spec_enum ("handle_position",
                                                       _("Handle position"),
-                                                      _("Position of the handle relative to the child widget."),
+                                                      _("Position of the handle relative to the child widget"),
 						      GTK_TYPE_POSITION_TYPE,
 						      GTK_POS_LEFT,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -208,10 +209,11 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
                                    PROP_SNAP_EDGE,
                                    g_param_spec_enum ("snap_edge",
                                                       _("Snap edge"),
-                                                      _("Side of the handlebox that's lined up with the docking point to dock the handlebox."),
+                                                      _("Side of the handlebox that's lined up with the docking point to dock the handlebox"),
 						      GTK_TYPE_POSITION_TYPE,
 						      GTK_POS_TOP,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
+
   object_class->destroy = gtk_handle_box_destroy;
 
   widget_class->map = gtk_handle_box_map;
@@ -234,21 +236,23 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
   class->child_detached = NULL;
 
   handle_box_signals[SIGNAL_CHILD_ATTACHED] =
-    gtk_signal_new ("child_attached",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkHandleBoxClass, child_attached),
-		    _gtk_marshal_VOID__OBJECT,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
+    g_signal_new ("child_attached",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkHandleBoxClass, child_attached),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_WIDGET);
   handle_box_signals[SIGNAL_CHILD_DETACHED] =
-    gtk_signal_new ("child_detached",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkHandleBoxClass, child_detached),
-		    _gtk_marshal_VOID__OBJECT,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
+    g_signal_new ("child_detached",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkHandleBoxClass, child_detached),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_WIDGET);
 }
 
 static void
@@ -322,7 +326,7 @@ gtk_handle_box_get_property (GObject         *object,
 GtkWidget*
 gtk_handle_box_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (gtk_handle_box_get_type ()));
+  return g_object_new (GTK_TYPE_HANDLE_BOX, NULL);
 }
 
 static void
@@ -405,11 +409,11 @@ gtk_handle_box_realize (GtkWidget *widget)
   attributes.width = widget->allocation.width;
   attributes.height = widget->allocation.height;
   attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.event_mask |= (gtk_widget_get_events (widget) |
-			    GDK_EXPOSURE_MASK |
-			    GDK_BUTTON1_MOTION_MASK |
-			    GDK_POINTER_MOTION_HINT_MASK |
-			    GDK_BUTTON_PRESS_MASK |
+  attributes.event_mask = (gtk_widget_get_events (widget) |
+			   GDK_EXPOSURE_MASK |
+			   GDK_BUTTON1_MOTION_MASK |
+			   GDK_POINTER_MOTION_HINT_MASK |
+			   GDK_BUTTON_PRESS_MASK |
 			    GDK_BUTTON_RELEASE_MASK);
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
   hb->bin_window = gdk_window_new (widget->window, &attributes, attributes_mask);
@@ -432,7 +436,8 @@ gtk_handle_box_realize (GtkWidget *widget)
 			   GDK_FOCUS_CHANGE_MASK |
 			   GDK_STRUCTURE_MASK);
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
-  hb->float_window = gdk_window_new (NULL, &attributes, attributes_mask);
+  hb->float_window = gdk_window_new (gtk_widget_get_root_window (widget),
+				     &attributes, attributes_mask);
   gdk_window_set_user_data (hb->float_window, widget);
   gdk_window_set_decorations (hb->float_window, 0);
   gdk_window_set_type_hint (hb->float_window, GDK_WINDOW_TYPE_HINT_TOOLBAR);
@@ -470,7 +475,7 @@ gtk_handle_box_style_set (GtkWidget *widget,
       !GTK_WIDGET_NO_WINDOW (widget))
     {
       gtk_style_set_background (widget->style, widget->window,
-widget->state);
+				widget->state);
       gtk_style_set_background (widget->style, hb->bin_window, widget->state);
       gtk_style_set_background (widget->style, hb->float_window, widget->state);
     }
@@ -817,7 +822,7 @@ gtk_handle_box_paint (GtkWidget      *widget,
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
 
-  gdk_window_get_size (hb->bin_window, &width, &height);
+  gdk_drawable_get_size (hb->bin_window, &width, &height);
   
   if (!event)
     gtk_paint_box (widget->style,
@@ -876,17 +881,7 @@ gtk_handle_box_paint (GtkWidget      *widget,
 			 event ? &event->area : area);
 
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
-    {
-      GdkRectangle child_area;
-
-      if (!event) /* we were called from draw() */
-	{
-	  if (gtk_widget_intersect (bin->child, area, &child_area))
-	    gtk_widget_draw (bin->child, &child_area);
-	}
-      else /* we were called from expose() */
-	(* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
-    }
+    (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
 }
 
 static gint
@@ -970,7 +965,7 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 	      
 	      gdk_window_get_deskrelative_origin (hb->bin_window, &desk_x, &desk_y);
 	      gdk_window_get_origin (hb->bin_window, &root_x, &root_y);
-	      gdk_window_get_size (hb->bin_window, &width, &height);
+	      gdk_drawable_get_size (hb->bin_window, &width, &height);
 	      
 	      hb->float_allocation.x = root_x - event->x_root;
 	      hb->float_allocation.y = root_y - event->y_root;
@@ -981,7 +976,7 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 	      hb->deskoff_y = desk_y - root_y;
 	      
 	      gdk_window_get_origin (widget->window, &root_x, &root_y);
-	      gdk_window_get_size (widget->window, &width, &height);
+	      gdk_drawable_get_size (widget->window, &width, &height);
 	      
 	      hb->attach_allocation.x = root_x;
 	      hb->attach_allocation.y = root_y;
@@ -989,7 +984,8 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 	      hb->attach_allocation.height = height;
 
 	      hb->in_drag = TRUE;
-	      fleur = gdk_cursor_new (GDK_FLEUR);
+	      fleur = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+						  GDK_FLEUR);
 	      if (gdk_pointer_grab (widget->window,
 				    FALSE,
 				    (GDK_BUTTON1_MOTION_MASK |
@@ -1002,7 +998,7 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 		  hb->in_drag = FALSE;
 		}
 	      
-	      gdk_cursor_destroy (fleur);
+	      gdk_cursor_unref (fleur);
 	      event_handled = TRUE;
 	    }
 	  else if (hb->child_detached) /* Double click */
@@ -1017,7 +1013,8 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
       if (event->window != widget->window)
 	return FALSE;
       
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
+      gdk_display_pointer_ungrab (gtk_widget_get_display (widget),
+				  GDK_CURRENT_TIME);
       hb->in_drag = FALSE;
       event_handled = TRUE;
     }
@@ -1046,7 +1043,8 @@ gtk_handle_box_motion (GtkWidget      *widget,
    */
   new_x = 0;
   new_y = 0;
-  gdk_window_get_pointer (NULL, &new_x, &new_y, NULL);
+  gdk_window_get_pointer (gtk_widget_get_root_window (widget), 
+			  &new_x, &new_y, NULL);
   new_x += hb->float_allocation.x;
   new_y += hb->float_allocation.y;
 
@@ -1118,9 +1116,10 @@ gtk_handle_box_motion (GtkWidget      *widget,
 	  gdk_window_hide (hb->float_window);
 	  gdk_window_reparent (hb->bin_window, widget->window, 0, 0);
 	  hb->float_window_mapped = FALSE;
-	  gtk_signal_emit (GTK_OBJECT (hb),
-			   handle_box_signals[SIGNAL_CHILD_ATTACHED],
-			   GTK_BIN (hb)->child);
+	  g_signal_emit (hb,
+			 handle_box_signals[SIGNAL_CHILD_ATTACHED],
+			 0,
+			 GTK_BIN (hb)->child);
 	  
 	  gtk_widget_queue_resize (widget);
 	}
@@ -1129,7 +1128,7 @@ gtk_handle_box_motion (GtkWidget      *widget,
     {
       gint width, height;
 
-      gdk_window_get_size (hb->float_window, &width, &height);
+      gdk_drawable_get_size (hb->float_window, &width, &height);
       new_x += hb->deskoff_x;
       new_y += hb->deskoff_y;
 
@@ -1189,13 +1188,14 @@ gtk_handle_box_motion (GtkWidget      *widget,
 	  /* this extra move is neccessary if we use decorations, or our
 	   * window manager insists on decorations.
 	   */
-	  gdk_flush ();
+	  gdk_display_sync (gtk_widget_get_display (widget));
 	  gdk_window_move (hb->float_window, new_x, new_y);
-	  gdk_flush ();
+	  gdk_display_sync (gtk_widget_get_display (widget));
 #endif	/* 0 */
-	  gtk_signal_emit (GTK_OBJECT (hb),
-			   handle_box_signals[SIGNAL_CHILD_DETACHED],
-			   GTK_BIN (hb)->child);
+	  g_signal_emit (hb,
+			 handle_box_signals[SIGNAL_CHILD_DETACHED],
+			 0,
+			 GTK_BIN (hb)->child);
 	  gtk_handle_box_draw_ghost (hb);
 	  
 	  gtk_widget_queue_resize (widget);
@@ -1241,25 +1241,29 @@ gtk_handle_box_delete_event (GtkWidget *widget,
 static void
 gtk_handle_box_reattach (GtkHandleBox *hb)
 {
+  GtkWidget *widget = GTK_WIDGET (hb);
+  
   if (hb->child_detached)
     {
       hb->child_detached = FALSE;
       if (GTK_WIDGET_REALIZED (hb))
 	{
 	  gdk_window_hide (hb->float_window);
-	  gdk_window_reparent (hb->bin_window, GTK_WIDGET (hb)->window, 0, 0);
+	  gdk_window_reparent (hb->bin_window, widget->window, 0, 0);
 
 	  if (GTK_BIN (hb)->child)
-	    gtk_signal_emit (GTK_OBJECT (hb),
-			     handle_box_signals[SIGNAL_CHILD_ATTACHED],
-			     GTK_BIN (hb)->child);
+	    g_signal_emit (hb,
+			   handle_box_signals[SIGNAL_CHILD_ATTACHED],
+			   0,
+			   GTK_BIN (hb)->child);
 
 	}
       hb->float_window_mapped = FALSE;
     }
   if (hb->in_drag)
     {
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
+      gdk_display_pointer_ungrab (gtk_widget_get_display (GTK_WIDGET (hb)),
+				  GDK_CURRENT_TIME);
       hb->in_drag = FALSE;
     }
 
