@@ -57,7 +57,6 @@
 
 #include "gtkmain.h"
 #include "gtkselection.h"
-#include "gtksignal.h"
 
 /* #define DEBUG_SELECTION */
 
@@ -69,6 +68,9 @@
 #else
 #define GTK_SELECTION_MAX_SIZE 4000
 #endif
+
+#define IDLE_ABORT_TIME 300
+
 enum {
   INCR,
   MULTIPLE,
@@ -84,9 +86,10 @@ typedef struct _GtkRetrievalInfo GtkRetrievalInfo;
 
 struct _GtkSelectionInfo
 {
-  GdkAtom    selection;
-  GtkWidget *widget;		/* widget that owns selection */
-  guint32    time;		/* time used to acquire selection */
+  GdkAtom	 selection;
+  GtkWidget	*widget;	/* widget that owns selection */
+  guint32	 time;		/* time used to acquire selection */
+  GdkDisplay	*display;	/* needed in gtk_selection_remove_all */    
 };
 
 struct _GtkIncrConversion 
@@ -102,7 +105,6 @@ struct _GtkIncrConversion
 
 struct _GtkIncrInfo
 {
-  GtkWidget *widget;		/* Selection owner */
   GdkWindow *requestor;		/* Requestor window - we create a GdkWindow
 				   so we can receive events */
   GdkAtom    selection;		/* Selection we're sending */
@@ -293,29 +295,32 @@ gtk_target_list_find (GtkTargetList *list,
   return FALSE;
 }
 
-
-/*************************************************************
- * gtk_selection_owner_set:
- *     Claim ownership of a selection.
- *   arguments:
- *     widget:		new selection owner
- *     selection:	which selection
- *     time:		time (use GDK_CURRENT_TIME only if necessary)
+/**
+ * gtk_selection_owner_set_for_display:
+ * @display: the #Gdkdisplay where the selection is set 
+ * @widget: new selection owner (a #GdkWidget), or %NULL.
+ * @selection: an interned atom representing the selection to claim.
+ * @time: timestamp with which to claim the selection
  *
- *   results:
- *************************************************************/
-
+ * Claim ownership of a given selection for a particular widget, or,
+ * if @widget is %NULL, release ownership of the selection.
+ *
+ * Return value: TRUE if the operation succeeded 
+ */
 gboolean
-gtk_selection_owner_set (GtkWidget *widget,
-			 GdkAtom    selection,
-			 guint32    time)
+gtk_selection_owner_set_for_display (GdkDisplay   *display,
+				     GtkWidget    *widget,
+				     GdkAtom       selection,
+				     guint32       time)
 {
   GList *tmp_list;
   GtkWidget *old_owner;
   GtkSelectionInfo *selection_info = NULL;
   GdkWindow *window;
 
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
   g_return_val_if_fail (widget == NULL || GTK_WIDGET_REALIZED (widget), FALSE);
+  g_return_val_if_fail (widget == NULL || gtk_widget_get_display (widget) == display, FALSE);
   
   if (widget == NULL)
     window = NULL;
@@ -334,7 +339,7 @@ gtk_selection_owner_set (GtkWidget *widget,
       tmp_list = tmp_list->next;
     }
   
-  if (gdk_selection_owner_set (window, selection, time, TRUE))
+  if (gdk_selection_owner_set_for_display (display, window, selection, time, TRUE))
     {
       old_owner = NULL;
       
@@ -357,7 +362,8 @@ gtk_selection_owner_set (GtkWidget *widget,
 	      selection_info->selection = selection;
 	      selection_info->widget = widget;
 	      selection_info->time = time;
-	      current_selections = g_list_prepend (current_selections, 
+	      selection_info->display = display;
+	      current_selections = g_list_prepend (current_selections,
 						   selection_info);
 	    }
 	  else
@@ -365,6 +371,7 @@ gtk_selection_owner_set (GtkWidget *widget,
 	      old_owner = selection_info->widget;
 	      selection_info->widget = widget;
 	      selection_info->time = time;
+	      selection_info->display = display;
 	    }
 	}
       /* If another widget in the application lost the selection,
@@ -372,14 +379,15 @@ gtk_selection_owner_set (GtkWidget *widget,
        */
       if (old_owner && old_owner != widget)
 	{
-	  GdkEventSelection event;
+	  GdkEvent *event = gdk_event_new (GDK_SELECTION_CLEAR);
 	  
-	  event.type = GDK_SELECTION_CLEAR;
-	  event.window = old_owner->window;
-	  event.selection = selection;
-	  event.time = time;
+	  event->selection.window = g_object_ref (old_owner->window);
+	  event->selection.selection = selection;
+	  event->selection.time = time;
 	  
-	  gtk_widget_event (old_owner, (GdkEvent *) &event);
+	  gtk_widget_event (old_owner, event);
+
+	  gdk_event_free (event);
 	}
       return TRUE;
     }
@@ -387,6 +395,40 @@ gtk_selection_owner_set (GtkWidget *widget,
     return FALSE;
 }
 
+/**
+ * gtk_selection_owner_set:
+ * @widget:  a #GtkWidget, or %NULL.
+ * @selection:  an interned atom representing the selection to claim
+ * @time: timestamp with which to claim the selection
+ * 
+ * Claims ownership of a given selection for a particular widget,
+ * or, if @widget is %NULL, release ownership of the selection.
+ * 
+ * Return value: %TRUE if the operation succeeded
+ **/
+gboolean
+gtk_selection_owner_set (GtkWidget *widget,
+			 GdkAtom    selection,
+			 guint32    time)
+{
+  GdkDisplay *display;
+  
+  g_return_val_if_fail (widget == NULL || GTK_WIDGET_REALIZED (widget), FALSE);
+
+  if (widget)
+    display = gtk_widget_get_display (widget);
+  else
+    {
+      GTK_NOTE (MULTIHEAD,
+		g_warning ("gtk_selection_owner_set (NULL,...) is not multihead safe"));
+		 
+      display = gdk_display_get_default ();
+    }
+  
+  return gtk_selection_owner_set_for_display (display, widget,
+					      selection, time);
+}
+
 /*************************************************************
  * gtk_selection_add_target
  *     Add specified target to list of supported targets
@@ -415,7 +457,7 @@ gtk_selection_target_list_get (GtkWidget    *widget,
   GList *tmp_list;
   GList *lists;
 
-  lists = gtk_object_get_data (GTK_OBJECT (widget), gtk_selection_handler_key);
+  lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
   tmp_list = lists;
   while (tmp_list)
@@ -431,7 +473,7 @@ gtk_selection_target_list_get (GtkWidget    *widget,
   sellist->list = gtk_target_list_new (NULL, 0);
 
   lists = g_list_prepend (lists, sellist);
-  gtk_object_set_data (GTK_OBJECT (widget), gtk_selection_handler_key, lists);
+  g_object_set_data (G_OBJECT (widget), gtk_selection_handler_key, lists);
 
   return sellist->list;
 }
@@ -443,7 +485,7 @@ gtk_selection_target_list_remove (GtkWidget    *widget)
   GList *tmp_list;
   GList *lists;
 
-  lists = gtk_object_get_data (GTK_OBJECT (widget), gtk_selection_handler_key);
+  lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
   tmp_list = lists;
   while (tmp_list)
@@ -457,7 +499,7 @@ gtk_selection_target_list_remove (GtkWidget    *widget)
     }
 
   g_list_free (lists);
-  gtk_object_set_data (GTK_OBJECT (widget), gtk_selection_handler_key, NULL);
+  g_object_set_data (G_OBJECT (widget), gtk_selection_handler_key, NULL);
 }
 
 /**
@@ -476,7 +518,7 @@ gtk_selection_clear_targets (GtkWidget *widget,
   GList *tmp_list;
   GList *lists;
 
-  lists = gtk_object_get_data (GTK_OBJECT (widget), gtk_selection_handler_key);
+  lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
   tmp_list = lists;
   while (tmp_list)
@@ -494,7 +536,7 @@ gtk_selection_clear_targets (GtkWidget *widget,
       tmp_list = tmp_list->next;
     }
   
-  gtk_object_set_data (GTK_OBJECT (widget), gtk_selection_handler_key, lists);
+  g_object_set_data (G_OBJECT (widget), gtk_selection_handler_key, lists);
 }
 
 void 
@@ -547,19 +589,6 @@ gtk_selection_remove_all (GtkWidget *widget)
   
   /* Remove pending requests/incrs for this widget */
   
-  tmp_list = current_incrs;
-  while (tmp_list)
-    {
-      next = tmp_list->next;
-      if (((GtkIncrInfo *)tmp_list->data)->widget == widget)
-	{
-	  current_incrs = g_list_remove_link (current_incrs, tmp_list);
-	  /* structure will be freed in timeout */
-	  g_list_free (tmp_list);
-	}
-      tmp_list = next;
-    }
-  
   tmp_list = current_retrievals;
   while (tmp_list)
     {
@@ -584,10 +613,11 @@ gtk_selection_remove_all (GtkWidget *widget)
       
       if (selection_info->widget == widget)
 	{	
-	  gdk_selection_owner_set (NULL, 
-				   selection_info->selection,
-				   GDK_CURRENT_TIME, FALSE);
-	  current_selections = g_list_remove_link (current_selections, 
+	  gdk_selection_owner_set_for_display (selection_info->display,
+					       NULL, 
+					       selection_info->selection,
+				               GDK_CURRENT_TIME, FALSE);
+	  current_selections = g_list_remove_link (current_selections,
 						   tmp_list);
 	  g_list_free (tmp_list);
 	  g_free (selection_info);
@@ -627,6 +657,7 @@ gtk_selection_convert (GtkWidget *widget,
   GtkRetrievalInfo *info;
   GList *tmp_list;
   GdkWindow *owner_window;
+  GdkDisplay *display;
   
   g_return_val_if_fail (widget != NULL, FALSE);
   
@@ -656,13 +687,15 @@ gtk_selection_convert (GtkWidget *widget,
   info->widget = widget;
   info->selection = selection;
   info->target = target;
+  info->idle_time = 0;
   info->buffer = NULL;
   info->offset = -1;
   
   /* Check if this process has current owner. If so, call handler
      procedure directly to avoid deadlocks with INCR. */
-  
-  owner_window = gdk_selection_owner_get (selection);
+
+  display = gtk_widget_get_display (widget);
+  owner_window = gdk_selection_owner_get_for_display (display, selection);
   
   if (owner_window != NULL)
     {
@@ -673,6 +706,7 @@ gtk_selection_convert (GtkWidget *widget,
       selection_data.target = target;
       selection_data.data = NULL;
       selection_data.length = -1;
+      selection_data.display = display;
       
       gdk_window_get_user_data (owner_window, (gpointer *)&owner_widget);
       
@@ -824,7 +858,8 @@ gtk_selection_data_set_text (GtkSelectionData     *selection_data,
       gint new_length;
 
       tmp = g_strndup (str, len);
-      if (gdk_utf8_to_compound_text (tmp, &encoding, &format, &text, &new_length))
+      if (gdk_utf8_to_compound_text_for_display (selection_data->display, tmp,
+						 &encoding, &format, &text, &new_length))
 	{
 	  gtk_selection_data_set (selection_data, encoding, format, text, new_length);
 	  gdk_free_compound_text (text);
@@ -863,11 +898,12 @@ gtk_selection_data_get_text (GtkSelectionData *selection_data)
     {
       gchar **list;
       gint i;
-      gint count = gdk_text_property_to_utf8_list (selection_data->type,
-						   selection_data->format, 
-						   selection_data->data,
-						   selection_data->length,
-						   &list);
+      gint count = gdk_text_property_to_utf8_list_for_display (selection_data->display,
+      							       selection_data->type,
+						   	       selection_data->format, 
+						               selection_data->data,
+						               selection_data->length,
+						               &list);
       if (count > 0)
 	result = list[0];
 
@@ -886,7 +922,7 @@ gtk_selection_data_get_text (GtkSelectionData *selection_data)
  *           stored here must be freed with g_free().
  * @n_atoms: location to store number of items in @targets.
  * 
- * Get the contents of @selection_data as an array of targets.
+ * Gets the contents of @selection_data as an array of targets.
  * This can be used to interpret the results of getting
  * the standard TARGETS target that is always supplied for
  * any selection.
@@ -926,7 +962,7 @@ gtk_selection_data_get_targets (GtkSelectionData  *selection_data,
  * @selection_data: a #GtkSelectionData object
  * 
  * Given a #GtkSelectionData object holding a list of targets,
- * Determines if any of the targets in @targets can be used to
+ * determines if any of the targets in @targets can be used to
  * provide text.
  * 
  * Return value: %TRUE if @selection_data holds a list of targets,
@@ -972,17 +1008,23 @@ gtk_selection_init (void)
   gtk_selection_atoms[MULTIPLE] = gdk_atom_intern ("MULTIPLE", FALSE);
   gtk_selection_atoms[TIMESTAMP] = gdk_atom_intern ("TIMESTAMP", FALSE);
   gtk_selection_atoms[TARGETS] = gdk_atom_intern ("TARGETS", FALSE);
+
+  initialize = FALSE;
 }
 
-/*************************************************************
+/**
  * gtk_selection_clear:
- *     Handler for "selection_clear_event"
- *   arguments:
- *     widget:
- *     event:
- *   results:
- *************************************************************/
-
+ * @widget: a #GtkWidget
+ * @event: the event
+ * 
+ * The default handler for the GtkWidget::selection_clear_event
+ * signal. Instead of calling this function, chain up from
+ * your selection_clear_event handler. Calling this function
+ * from any other context is illegal. This function will
+ * be deprecated in future versions of GTK+.
+ * 
+ * Return value: %TRUE if the event was handled, otherwise false
+ **/
 gboolean
 gtk_selection_clear (GtkWidget         *widget,
 		     GdkEventSelection *event)
@@ -1018,7 +1060,7 @@ gtk_selection_clear (GtkWidget         *widget,
 
 
 /*************************************************************
- * gtk_selection_request:
+ * _gtk_selection_request:
  *     Handler for "selection_request_event" 
  *   arguments:
  *     widget:
@@ -1027,8 +1069,8 @@ gtk_selection_clear (GtkWidget         *widget,
  *************************************************************/
 
 gboolean
-gtk_selection_request (GtkWidget *widget,
-		       GdkEventSelection *event)
+_gtk_selection_request (GtkWidget *widget,
+			GdkEventSelection *event)
 {
   GtkIncrInfo *info;
   GList *tmp_list;
@@ -1056,15 +1098,19 @@ gtk_selection_request (GtkWidget *widget,
     return FALSE;
   
   info = g_new (GtkIncrInfo, 1);
+
+  g_object_ref (widget);
   
-  info->widget = widget;
   info->selection = event->selection;
   info->num_incrs = 0;
   
   /* Create GdkWindow structure for the requestor */
-  info->requestor = gdk_window_lookup (event->requestor);
+  
+  info->requestor = gdk_window_lookup_for_display (gtk_widget_get_display (widget),
+						   event->requestor);
   if (!info->requestor)
-    info->requestor = gdk_window_foreign_new (event->requestor);
+    info->requestor = gdk_window_foreign_new_for_display (gtk_widget_get_display (widget),
+							  event->requestor);
   
   /* Determine conversions we need to perform */
   
@@ -1081,8 +1127,12 @@ gtk_selection_request (GtkWidget *widget,
 			     0, GTK_SELECTION_MAX_SIZE, FALSE,
 			     &type, &format, &length, &mult_atoms))
 	{
-	  gdk_selection_send_notify (event->requestor, event->selection,
-				     event->target, GDK_NONE, event->time);
+	  gdk_selection_send_notify_for_display (gtk_widget_get_display (widget),
+						 event->requestor, 
+						 event->selection,
+						 event->target, 
+						 GDK_NONE, 
+						 event->time);
 	  g_free (mult_atoms);
 	  g_free (info);
 	  return TRUE;
@@ -1118,6 +1168,7 @@ gtk_selection_request (GtkWidget *widget,
       data.target = info->conversions[i].target;
       data.data = NULL;
       data.length = -1;
+      data.display = gtk_widget_get_display (widget);
       
 #ifdef DEBUG_SELECTION
       g_message ("Selection %ld, target %ld (%s) requested by 0x%x (property = %ld)",
@@ -1193,7 +1244,7 @@ gtk_selection_request (GtkWidget *widget,
   if (event->target == gtk_selection_atoms[MULTIPLE])
     {
       gdk_property_change (info->requestor, event->property,
-			   GDK_SELECTION_TYPE_ATOM, 32, 
+			   gdk_atom_intern ("ATOM_PAIR", FALSE), 32, 
 			   GDK_PROP_MODE_REPLACE,
 			   mult_atoms, 2*info->num_conversions);
       g_free (mult_atoms);
@@ -1203,26 +1254,36 @@ gtk_selection_request (GtkWidget *widget,
       info->conversions[0].property == GDK_NONE)
     {
       /* Reject the entire conversion */
-      gdk_selection_send_notify (event->requestor, event->selection, 
-				 event->target, GDK_NONE, event->time);
+      gdk_selection_send_notify_for_display (gtk_widget_get_display (widget),
+					     event->requestor, 
+					     event->selection, 
+					     event->target, 
+					     GDK_NONE, 
+					     event->time);
     }
   else
     {
-      gdk_selection_send_notify (event->requestor, event->selection, 
-				 event->target, event->property, event->time);
+      gdk_selection_send_notify_for_display (gtk_widget_get_display (widget),
+					     event->requestor, 
+					     event->selection,
+					     event->target,
+					     event->property, 
+					     event->time);
     }
-  
+
   if (info->num_incrs == 0)
     {
       g_free (info->conversions);
       g_free (info);
     }
+
+  g_object_unref (widget);
   
   return TRUE;
 }
 
 /*************************************************************
- * gtk_selection_incr_event:
+ * _gtk_selection_incr_event:
  *     Called whenever an PropertyNotify event occurs for an 
  *     GdkWindow with user_data == NULL. These will be notifications
  *     that a window we are sending the selection to via the
@@ -1237,8 +1298,8 @@ gtk_selection_request (GtkWidget *widget,
  *************************************************************/
 
 gboolean
-gtk_selection_incr_event (GdkWindow	   *window,
-			  GdkEventProperty *event)
+_gtk_selection_incr_event (GdkWindow	   *window,
+			   GdkEventProperty *event)
 {
   GList *tmp_list;
   GtkIncrInfo *info = NULL;
@@ -1369,9 +1430,9 @@ gtk_selection_incr_timeout (GtkIncrInfo *info)
     }
   
   /* If retrieval is finished */
-  if (!tmp_list || info->idle_time >= 5)
+  if (!tmp_list || info->idle_time >= IDLE_ABORT_TIME)
     {
-      if (tmp_list && info->idle_time >= 5)
+      if (tmp_list && info->idle_time >= IDLE_ABORT_TIME)
 	{
 	  current_incrs = g_list_remove_link (current_incrs, tmp_list);
 	  g_list_free (tmp_list);
@@ -1398,7 +1459,7 @@ gtk_selection_incr_timeout (GtkIncrInfo *info)
 }
 
 /*************************************************************
- * gtk_selection_notify:
+ * _gtk_selection_notify:
  *     Handler for "selection_notify_event" signals on windows
  *     where a retrieval is currently in process. The selection
  *     owner has responded to our conversion request.
@@ -1411,8 +1472,8 @@ gtk_selection_incr_timeout (GtkIncrInfo *info)
  *************************************************************/
 
 gboolean
-gtk_selection_notify (GtkWidget	       *widget,
-		      GdkEventSelection *event)
+_gtk_selection_notify (GtkWidget	       *widget,
+		       GdkEventSelection *event)
 {
   GList *tmp_list;
   GtkRetrievalInfo *info = NULL;
@@ -1487,7 +1548,7 @@ gtk_selection_notify (GtkWidget	       *widget,
 }
 
 /*************************************************************
- * gtk_selection_property_notify:
+ * _gtk_selection_property_notify:
  *     Handler for "property_notify_event" signals on windows
  *     where a retrieval is currently in process. The selection
  *     owner has added more data.
@@ -1500,8 +1561,8 @@ gtk_selection_notify (GtkWidget	       *widget,
  *************************************************************/
 
 gboolean
-gtk_selection_property_notify (GtkWidget	*widget,
-			       GdkEventProperty *event)
+_gtk_selection_property_notify (GtkWidget	*widget,
+				GdkEventProperty *event)
 {
   GList *tmp_list;
   GtkRetrievalInfo *info = NULL;
@@ -1619,9 +1680,9 @@ gtk_selection_retrieval_timeout (GtkRetrievalInfo *info)
     }
   
   /* If retrieval is finished */
-  if (!tmp_list || info->idle_time >= 5)
+  if (!tmp_list || info->idle_time >= IDLE_ABORT_TIME)
     {
-      if (tmp_list && info->idle_time >= 5)
+      if (tmp_list && info->idle_time >= IDLE_ABORT_TIME)
 	{
 	  current_retrievals = g_list_remove_link (current_retrievals, tmp_list);
 	  g_list_free (tmp_list);
@@ -1670,10 +1731,11 @@ gtk_selection_retrieval_report (GtkRetrievalInfo *info,
   
   data.length = length;
   data.data = buffer;
+  data.display = gtk_widget_get_display (info->widget);
   
-  gtk_signal_emit_by_name (GTK_OBJECT(info->widget),
-			   "selection_received", 
-			   &data, time);
+  g_signal_emit_by_name (info->widget,
+			 "selection_received", 
+			 &data, time);
 }
 
 /*************************************************************
@@ -1706,10 +1768,10 @@ gtk_selection_invoke_handler (GtkWidget	       *widget,
   if (target_list && 
       gtk_target_list_find (target_list, data->target, &info))
     {
-      gtk_signal_emit_by_name (GTK_OBJECT (widget), 
-			       "selection_get",
-			       data,
-			       info, time);
+      g_signal_emit_by_name (widget,
+			     "selection_get",
+			     data,
+			     info, time);
     }
   else
     gtk_selection_default_handler (widget, data);
@@ -1830,6 +1892,19 @@ gtk_selection_data_free (GtkSelectionData *data)
   g_free (data);
 }
 
+GType
+gtk_selection_data_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("GtkSelectionData",
+					     (GBoxedCopyFunc) gtk_selection_data_copy,
+					     (GBoxedFreeFunc) gtk_selection_data_free);
+
+  return our_type;
+}
+
 static int 
 gtk_selection_bytes_per_item (gint format)
 {
