@@ -25,6 +25,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <string.h>
+
 #include "gtkbutton.h"
 #include "gtktogglebutton.h"
 #include "gtkradiobutton.h"
@@ -45,6 +47,7 @@
 #define DEFAULT_SPACE_STYLE GTK_TOOLBAR_SPACE_LINE
 
 #define DEFAULT_ICON_SIZE GTK_ICON_SIZE_LARGE_TOOLBAR
+#define DEFAULT_TOOLBAR_STYLE GTK_TOOLBAR_BOTH
 
 #define SPACE_LINE_DIVISION 10
 #define SPACE_LINE_START    3
@@ -53,7 +56,7 @@
 enum {
   PROP_0,
   PROP_ORIENTATION,
-  PROP_TOOLBAR_STYLE,
+  PROP_TOOLBAR_STYLE
 };
 
 enum {
@@ -72,7 +75,6 @@ struct _GtkToolbarChildSpace
 
 static void gtk_toolbar_class_init               (GtkToolbarClass *class);
 static void gtk_toolbar_init                     (GtkToolbar      *toolbar);
-static void gtk_toolbar_finalize                 (GObject         *object);
 static void gtk_toolbar_set_property             (GObject         *object,
 						  guint            prop_id,
 						  const GValue    *value,
@@ -92,6 +94,8 @@ static void gtk_toolbar_style_set                (GtkWidget       *widget,
                                                   GtkStyle        *prev_style);
 static gboolean gtk_toolbar_focus                (GtkWidget       *widget,
                                                   GtkDirectionType dir);
+static void gtk_toolbar_hierarchy_changed        (GtkWidget       *widget,
+						  GtkWidget       *previous_toplevel);
 static void gtk_toolbar_show_all                 (GtkWidget       *widget);
 static void gtk_toolbar_add                      (GtkContainer    *container,
 				                  GtkWidget       *widget);
@@ -116,8 +120,7 @@ static GtkWidget * gtk_toolbar_internal_insert_element (GtkToolbar          *too
                                                         GtkWidget           *icon,
                                                         GtkSignalFunc        callback,
                                                         gpointer             user_data,
-                                                        gint                 position,
-                                                        gboolean             has_mnemonic);
+                                                        gint                 position);
 
 static GtkWidget * gtk_toolbar_internal_insert_item (GtkToolbar    *toolbar,
                                                      const char    *text,
@@ -126,8 +129,7 @@ static GtkWidget * gtk_toolbar_internal_insert_item (GtkToolbar    *toolbar,
                                                      GtkWidget     *icon,
                                                      GtkSignalFunc  callback,
                                                      gpointer       user_data,
-                                                     gint           position,
-                                                     gboolean       has_mnemonic);
+                                                     gint           position);
 
 static void        gtk_toolbar_update_button_relief (GtkToolbar *toolbar);
 
@@ -141,26 +143,28 @@ static GtkContainerClass *parent_class;
 static guint toolbar_signals[LAST_SIGNAL] = { 0 };
 
 
-GtkType
+GType
 gtk_toolbar_get_type (void)
 {
-  static GtkType toolbar_type = 0;
+  static GType toolbar_type = 0;
 
   if (!toolbar_type)
     {
-      static const GtkTypeInfo toolbar_info =
+      static const GTypeInfo toolbar_info =
       {
-	"GtkToolbar",
-	sizeof (GtkToolbar),
 	sizeof (GtkToolbarClass),
-	(GtkClassInitFunc) gtk_toolbar_class_init,
-	(GtkObjectInitFunc) gtk_toolbar_init,
-	/* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_toolbar_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkToolbar),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_toolbar_init,
       };
 
-      toolbar_type = gtk_type_unique (gtk_container_get_type (), &toolbar_info);
+      toolbar_type = g_type_register_static (GTK_TYPE_CONTAINER, "GtkToolbar",
+					     &toolbar_info, 0);
     }
 
   return toolbar_type;
@@ -179,10 +183,8 @@ gtk_toolbar_class_init (GtkToolbarClass *class)
   widget_class = (GtkWidgetClass *) class;
   container_class = (GtkContainerClass *) class;
 
-  parent_class = gtk_type_class (gtk_container_get_type ());
+  parent_class = g_type_class_peek_parent (class);
 
-  gobject_class->finalize = gtk_toolbar_finalize;
-  
   object_class->destroy = gtk_toolbar_destroy;
   gobject_class->set_property = gtk_toolbar_set_property;
   gobject_class->get_property = gtk_toolbar_get_property;
@@ -193,6 +195,7 @@ gtk_toolbar_class_init (GtkToolbarClass *class)
   widget_class->style_set = gtk_toolbar_style_set;
   widget_class->show_all = gtk_toolbar_show_all;
   widget_class->focus = gtk_toolbar_focus;
+  widget_class->hierarchy_changed = gtk_toolbar_hierarchy_changed;
   
   container_class->add = gtk_toolbar_add;
   container_class->remove = gtk_toolbar_remove;
@@ -202,21 +205,23 @@ gtk_toolbar_class_init (GtkToolbarClass *class)
   class->style_changed = gtk_real_toolbar_style_changed;
 
   toolbar_signals[ORIENTATION_CHANGED] =
-    gtk_signal_new ("orientation_changed",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkToolbarClass, orientation_changed),
-		    _gtk_marshal_VOID__ENUM,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_ORIENTATION);
+    g_signal_new ("orientation_changed",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkToolbarClass, orientation_changed),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__ENUM,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_ORIENTATION);
   toolbar_signals[STYLE_CHANGED] =
-    gtk_signal_new ("style_changed",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkToolbarClass, style_changed),
-		    _gtk_marshal_VOID__ENUM,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_TOOLBAR_STYLE);
+    g_signal_new ("style_changed",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkToolbarClass, style_changed),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__ENUM,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_TOOLBAR_STYLE);
   
   g_object_class_install_property (gobject_class,
 				   PROP_ORIENTATION,
@@ -284,26 +289,20 @@ gtk_toolbar_class_init (GtkToolbarClass *class)
                                                     _("Toolbar style"),
                                                     _("Whether default toolbars have text only, text and icons, icons only, etc."),
                                                     GTK_TYPE_TOOLBAR_STYLE,
-                                                    GTK_TOOLBAR_BOTH,
+                                                    DEFAULT_TOOLBAR_STYLE,
                                                     G_PARAM_READWRITE));
 
   gtk_settings_install_property (g_param_spec_enum ("gtk-toolbar-icon-size",
                                                     _("Toolbar icon size"),
                                                     _("Size of icons in default toolbars"),
                                                     GTK_TYPE_ICON_SIZE,
-                                                    GTK_ICON_SIZE_LARGE_TOOLBAR,
+                                                    DEFAULT_ICON_SIZE,
                                                     G_PARAM_READWRITE));  
 }
 
 static void
-style_change_notify (GObject    *object,
-                     GParamSpec *pspec,
-                     gpointer    data)
+style_change_notify (GtkToolbar *toolbar)
 {
-  GtkToolbar *toolbar;
-
-  toolbar = GTK_TOOLBAR (data);
-
   if (!toolbar->style_set)
     {
       /* pretend it was set, then unset, thus reverting to new default */
@@ -313,14 +312,8 @@ style_change_notify (GObject    *object,
 }
 
 static void
-icon_size_change_notify (GObject    *object,
-                         GParamSpec *pspec,
-                         gpointer    data)
+icon_size_change_notify (GtkToolbar *toolbar)
 {
-  GtkToolbar *toolbar;
-
-  toolbar = GTK_TOOLBAR (data);
-
   if (!toolbar->icon_size_set)
     {
       /* pretend it was set, then unset, thus reverting to new default */
@@ -329,6 +322,80 @@ icon_size_change_notify (GObject    *object,
     }
 }
 
+static GtkSettings *
+toolbar_get_settings (GtkToolbar *toolbar)
+{
+  return g_object_get_data (G_OBJECT (toolbar), "gtk-toolbar-settings");
+}
+
+static void
+toolbar_screen_changed (GtkToolbar *toolbar)
+{
+  GtkSettings *old_settings = toolbar_get_settings (toolbar);
+  GtkSettings *settings;
+
+  if (gtk_widget_has_screen (GTK_WIDGET (toolbar)))
+    settings = gtk_widget_get_settings (GTK_WIDGET (toolbar));
+  else
+    settings = NULL;
+
+  if (settings == old_settings)
+    return;
+
+  if (old_settings)
+    {
+      g_signal_handler_disconnect (old_settings, toolbar->style_set_connection);
+      g_signal_handler_disconnect (old_settings, toolbar->icon_size_connection);
+
+      g_object_unref (old_settings);
+    }
+  
+  if (settings)
+    {
+      toolbar->style_set_connection =
+	g_signal_connect_swapped (settings,
+				  "notify::gtk-toolbar-style",
+				  G_CALLBACK (style_change_notify),
+				  toolbar);
+      
+      toolbar->icon_size_connection =
+	g_signal_connect_swapped (settings,
+				  "notify::gtk-toolbar-icon-size",
+				   G_CALLBACK (icon_size_change_notify),
+				   toolbar);
+
+
+      g_object_ref (settings);
+      g_object_set_data (G_OBJECT (toolbar), "gtk-toolbar-settings", settings);
+    }
+  else
+    g_object_set_data (G_OBJECT (toolbar), "gtk-toolbar-settings", NULL);
+
+  style_change_notify (toolbar);
+  icon_size_change_notify (toolbar);
+}
+
+static void
+gtk_toolbar_hierarchy_changed (GtkWidget *widget,
+			       GtkWidget *previous_toplevel)
+{
+  GtkWidget *toplevel;
+  
+  if (previous_toplevel)
+    g_signal_handlers_disconnect_by_func (previous_toplevel,
+					  toolbar_screen_changed,
+					  widget);
+
+  toplevel = gtk_widget_get_toplevel (widget);
+  if (GTK_WIDGET_TOPLEVEL (toplevel))
+    g_signal_connect_swapped (toplevel,
+			      "notify::screen",
+			      G_CALLBACK (toolbar_screen_changed),
+			      widget);
+  
+  toolbar_screen_changed (GTK_TOOLBAR (widget));
+}
+
 static void
 gtk_toolbar_init (GtkToolbar *toolbar)
 {
@@ -339,6 +406,7 @@ gtk_toolbar_init (GtkToolbar *toolbar)
   toolbar->children     = NULL;
   toolbar->orientation  = GTK_ORIENTATION_HORIZONTAL;
   toolbar->icon_size    = DEFAULT_ICON_SIZE;
+  toolbar->style        = DEFAULT_TOOLBAR_STYLE;
   toolbar->tooltips     = gtk_tooltips_new ();
   g_object_ref (toolbar->tooltips);
   gtk_object_sink (GTK_OBJECT (toolbar->tooltips));
@@ -348,39 +416,6 @@ gtk_toolbar_init (GtkToolbar *toolbar)
 
   toolbar->style_set = FALSE;
   toolbar->icon_size_set = FALSE;
-  g_object_get (gtk_settings_get_default (),
-                "gtk-toolbar-icon-size",
-                &toolbar->icon_size,
-                "gtk-toolbar-style",
-                &toolbar->style,
-                NULL);
-  
-  toolbar->style_set_connection =
-    g_signal_connect (G_OBJECT (gtk_settings_get_default ()),
-		      "notify::gtk-toolbar-style",
-		      G_CALLBACK (style_change_notify),
-		      toolbar);
-
-  toolbar->icon_size_connection =
-    g_signal_connect (G_OBJECT (gtk_settings_get_default ()),
-		      "notify::gtk-toolbar-icon-size",
-		      G_CALLBACK (icon_size_change_notify),
-		      toolbar);
-}
-
-static void
-gtk_toolbar_finalize (GObject *object)
-{
-  GtkToolbar *toolbar;
-
-  toolbar = GTK_TOOLBAR (object);
-
-  g_signal_handler_disconnect (G_OBJECT (gtk_settings_get_default ()),
-                               toolbar->style_set_connection);
-  g_signal_handler_disconnect (G_OBJECT (gtk_settings_get_default ()),
-                               toolbar->icon_size_connection);
-  
-  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -429,7 +464,7 @@ gtk_toolbar_new (void)
 {
   GtkToolbar *toolbar;
 
-  toolbar = gtk_type_new (gtk_toolbar_get_type ());
+  toolbar = g_object_new (GTK_TYPE_TOOLBAR, NULL);
 
   return GTK_WIDGET (toolbar);
 }
@@ -446,7 +481,7 @@ gtk_toolbar_destroy (GtkObject *object)
 
   if (toolbar->tooltips)
     {
-      gtk_object_unref (GTK_OBJECT (toolbar->tooltips));
+      g_object_unref (toolbar->tooltips);
       toolbar->tooltips = NULL;
     }
 
@@ -458,10 +493,10 @@ gtk_toolbar_destroy (GtkObject *object)
 
       if (child->type != GTK_TOOLBAR_CHILD_SPACE)
 	{
-	  gtk_widget_ref (child->widget);
+	  g_object_ref (child->widget);
 	  gtk_widget_unparent (child->widget);
 	  gtk_widget_destroy (child->widget);
-	  gtk_widget_unref (child->widget);
+	  g_object_unref (child->widget);
 	}
 
       g_free (child);
@@ -657,8 +692,8 @@ gtk_toolbar_size_request (GtkWidget      *widget,
   /* Extra spacing */
   gtk_widget_style_get (widget, "internal_padding", &ipadding, NULL);
   
-  requisition->width += 2 * (widget->style->xthickness + ipadding);
-  requisition->height += 2 * (widget->style->ythickness + ipadding);
+  requisition->width += 2 * ipadding;
+  requisition->height += 2 * ipadding;
   
   toolbar->button_maxw = button_maxw;
   toolbar->button_maxh = button_maxh;
@@ -677,6 +712,8 @@ gtk_toolbar_size_allocate (GtkWidget     *widget,
   gint x_border_width, y_border_width;
   gint space_size;
   gint ipadding;
+  GtkTextDirection direction;
+  gint ltr_x = 0;		/* Quiet GCC */
   
   g_return_if_fail (GTK_IS_TOOLBAR (widget));
   g_return_if_fail (allocation != NULL);
@@ -684,16 +721,18 @@ gtk_toolbar_size_allocate (GtkWidget     *widget,
   toolbar = GTK_TOOLBAR (widget);
   widget->allocation = *allocation;
   
+  direction = gtk_widget_get_direction (widget);
+
   x_border_width = GTK_CONTAINER (toolbar)->border_width;
   y_border_width = GTK_CONTAINER (toolbar)->border_width;
 
   gtk_widget_style_get (widget, "internal_padding", &ipadding, NULL);
   
-  x_border_width += 2 * (widget->style->xthickness + ipadding);
-  y_border_width += 2 * (widget->style->ythickness + ipadding);
+  x_border_width += ipadding;
+  y_border_width += ipadding;
   
   if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL)
-    alloc.x = allocation->x + x_border_width;
+    ltr_x = allocation->x + x_border_width;
   else
     alloc.y = allocation->y + y_border_width;
 
@@ -711,9 +750,12 @@ gtk_toolbar_size_allocate (GtkWidget     *widget,
 
 	  if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL)
 	    {
-	      child_space->alloc_x = alloc.x;
+	      if (direction == GTK_TEXT_DIR_LTR)
+		child_space->alloc_x = ltr_x;
+	      else
+		child_space->alloc_x = allocation->width - ltr_x - space_size;
 	      child_space->alloc_y = allocation->y + (allocation->height - toolbar->button_maxh) / 2;
-	      alloc.x += space_size;
+	      ltr_x += space_size;
 	    }
 	  else
 	    {
@@ -733,15 +775,21 @@ gtk_toolbar_size_allocate (GtkWidget     *widget,
 	  alloc.width = toolbar->button_maxw;
 	  alloc.height = toolbar->button_maxh;
 
-	  if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL)
-	    alloc.y = allocation->y + (allocation->height - toolbar->button_maxh) / 2;
+	  if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL) 
+	    {
+	      if (direction == GTK_TEXT_DIR_LTR)
+		alloc.x = ltr_x;
+	      else
+		alloc.x = allocation->width - ltr_x - alloc.width;
+	      alloc.y = allocation->y + (allocation->height - toolbar->button_maxh) / 2;
+	    }
 	  else
 	    alloc.x = allocation->x + (allocation->width - toolbar->button_maxw) / 2;
-
+	  
 	  gtk_widget_size_allocate (child->widget, &alloc);
 
 	  if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL)
-	    alloc.x += toolbar->button_maxw;
+	    ltr_x += toolbar->button_maxw;
 	  else
 	    alloc.y += toolbar->button_maxh;
 
@@ -756,15 +804,21 @@ gtk_toolbar_size_allocate (GtkWidget     *widget,
 	  alloc.width = child_requisition.width;
 	  alloc.height = child_requisition.height;
 
-	  if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL)
-	    alloc.y = allocation->y + (allocation->height - child_requisition.height) / 2;
+	  if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL) 
+	    {
+	      if (direction == GTK_TEXT_DIR_LTR)
+		alloc.x = ltr_x;
+	      else
+		alloc.x = allocation->width - ltr_x - alloc.width;
+	      alloc.y = allocation->y + (allocation->height - child_requisition.height) / 2;
+	    }
 	  else
 	    alloc.x = allocation->x + (allocation->width - child_requisition.width) / 2;
 
 	  gtk_widget_size_allocate (child->widget, &alloc);
 
 	  if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL)
-	    alloc.x += child_requisition.width;
+	    ltr_x += child_requisition.width;
 	  else
 	    alloc.y += child_requisition.height;
 
@@ -924,14 +978,13 @@ gtk_toolbar_internal_insert_item (GtkToolbar    *toolbar,
                                   GtkWidget     *icon,
                                   GtkSignalFunc  callback,
                                   gpointer       user_data,
-                                  gint           position,
-                                  gboolean       has_mnemonic)
+                                  gint           position)
 {
   return gtk_toolbar_internal_insert_element (toolbar, GTK_TOOLBAR_CHILD_BUTTON,
                                               NULL, text,
                                               tooltip_text, tooltip_private_text,
                                               icon, callback, user_data,
-                                              position, has_mnemonic);
+                                              position);
 }
      
 GtkWidget *
@@ -947,7 +1000,7 @@ gtk_toolbar_insert_item (GtkToolbar    *toolbar,
   return gtk_toolbar_internal_insert_item (toolbar, 
                                            text, tooltip_text, tooltip_private_text,
                                            icon, callback, user_data,
-                                           position, FALSE);
+                                           position);
 }
 
 /**
@@ -981,7 +1034,9 @@ gtk_toolbar_set_icon_size (GtkToolbar  *toolbar,
   for (children = toolbar->children; children; children = children->next)
     {
       child = children->data;
-      if (child->type == GTK_TOOLBAR_CHILD_BUTTON &&
+      if ((child->type == GTK_TOOLBAR_CHILD_BUTTON ||
+	   child->type == GTK_TOOLBAR_CHILD_TOGGLEBUTTON ||
+	   child->type == GTK_TOOLBAR_CHILD_RADIOBUTTON) &&
 	  GTK_IS_IMAGE (child->icon))
 	{
 	  image = GTK_IMAGE (child->icon);
@@ -1011,7 +1066,7 @@ gtk_toolbar_set_icon_size (GtkToolbar  *toolbar,
 GtkIconSize
 gtk_toolbar_get_icon_size (GtkToolbar *toolbar)
 {
-  g_return_val_if_fail (GTK_IS_TOOLBAR (toolbar), GTK_ICON_SIZE_LARGE_TOOLBAR);
+  g_return_val_if_fail (GTK_IS_TOOLBAR (toolbar), DEFAULT_ICON_SIZE);
 
   return toolbar->icon_size;
 }
@@ -1030,9 +1085,14 @@ gtk_toolbar_unset_icon_size (GtkToolbar  *toolbar)
 
   if (toolbar->icon_size_set)
     {
-      g_object_get (gtk_settings_get_default (),
-                    "gtk-toolbar-icon-size",
-                    &size, NULL);
+      GtkSettings *settings = toolbar_get_settings (toolbar);
+
+      if (settings)
+	g_object_get (settings,
+		      "gtk-toolbar-icon-size", &size,
+		      NULL);
+      else
+	size = DEFAULT_ICON_SIZE;
 
       if (size != toolbar->icon_size)
         gtk_toolbar_set_icon_size (toolbar, size);
@@ -1041,6 +1101,32 @@ gtk_toolbar_unset_icon_size (GtkToolbar  *toolbar)
     }
 }
 
+static gchar *
+elide_underscores (const gchar *original)
+{
+  gchar *q, *result;
+  const gchar *p;
+  gboolean last_underscore;
+
+  q = result = g_malloc (strlen (original + 1));
+  last_underscore = FALSE;
+  
+  for (p = original; *p; p++)
+    {
+      if (!last_underscore && *p == '_')
+	last_underscore = TRUE;
+      else
+	{
+	  last_underscore = FALSE;
+	  *q++ = *p;
+	}
+    }
+  
+  *q = '\0';
+  
+  return result;
+}
+
 /**
  * gtk_toolbar_insert_stock:
  * @toolbar: A #GtkToolbar
@@ -1054,8 +1140,7 @@ gtk_toolbar_unset_icon_size (GtkToolbar  *toolbar)
  *
  * Inserts a stock item at the specified position of the toolbar.  If
  * @stock_id is not a known stock item ID, it's inserted verbatim,
- * except that underscores are used to mark mnemonics (see
- * gtk_label_new_with_mnemonic()).
+ * except that underscores used to mark mnemonics are removed.
  *
  * Returns: the inserted widget
  */
@@ -1069,32 +1154,33 @@ gtk_toolbar_insert_stock (GtkToolbar      *toolbar,
 			  gint             position)
 {
   GtkStockItem item;
-  GtkWidget *image;
+  GtkWidget *image = NULL;
+  const gchar *label;
+  gchar *label_no_mnemonic;
+  GtkWidget *retval;
 
   if (gtk_stock_lookup (stock_id, &item))
     {
       image = gtk_image_new_from_stock (stock_id, toolbar->icon_size);
-
-      return gtk_toolbar_internal_insert_item (toolbar,
-                                               item.label,
-                                               tooltip_text,
-                                               tooltip_private_text,
-                                               image,
-                                               callback,
-                                               user_data,
-                                               position,
-                                               TRUE);
+      label = item.label;
     }
   else
-    return gtk_toolbar_internal_insert_item (toolbar,
-                                             stock_id,
-                                             tooltip_text,
-                                             tooltip_private_text,
-                                             NULL,
-                                             callback,
-                                             user_data,
-                                             position,
-                                             TRUE);
+    label = stock_id;
+
+  label_no_mnemonic = elide_underscores (label);
+  
+  retval =  gtk_toolbar_internal_insert_item (toolbar,
+	   				      label_no_mnemonic,
+					      tooltip_text,
+					      tooltip_private_text,
+					      image,
+					      callback,
+					      user_data,
+					      position);
+
+  g_free (label_no_mnemonic);
+
+  return retval;
 }
 
 
@@ -1130,6 +1216,13 @@ gtk_toolbar_insert_space (GtkToolbar *toolbar,
 			      position);
 }
 
+/**
+ * gtk_toolbar_remove_space:
+ * @toolbar: a #GtkToolbar.
+ * @position: the index of the space to remove.
+ * 
+ * Removes a space from the specified position.
+ **/
 void
 gtk_toolbar_remove_space (GtkToolbar *toolbar,
                           gint        position)
@@ -1170,6 +1263,15 @@ gtk_toolbar_remove_space (GtkToolbar *toolbar,
   g_warning ("Toolbar position %d doesn't exist", position);
 }
 
+/**
+ * gtk_toolbar_append_widget:
+ * @toolbar: a #GtkToolbar.
+ * @widget: a #GtkWidget to add to the toolbar. 
+ * @tooltip_text: the element's tooltip.
+ * @tooltip_private_text: used for context-sensitive help about this toolbar element.
+ * 
+ * Adds a widget to the end of the given toolbar.
+ **/ 
 void
 gtk_toolbar_append_widget (GtkToolbar  *toolbar,
 			   GtkWidget   *widget,
@@ -1183,6 +1285,15 @@ gtk_toolbar_append_widget (GtkToolbar  *toolbar,
 			      toolbar->num_children);
 }
 
+/**
+ * gtk_toolbar_prepend_widget:
+ * @toolbar: a #GtkToolbar.
+ * @widget: a #GtkWidget to add to the toolbar. 
+ * @tooltip_text: the element's tooltip.
+ * @tooltip_private_text: used for context-sensitive help about this toolbar element.
+ * 
+ * Adds a widget to the beginning of the given toolbar.
+ **/ 
 void
 gtk_toolbar_prepend_widget (GtkToolbar  *toolbar,
 			    GtkWidget   *widget,
@@ -1196,6 +1307,16 @@ gtk_toolbar_prepend_widget (GtkToolbar  *toolbar,
 			      0);
 }
 
+/**
+ * gtk_toolbar_insert_widget:
+ * @toolbar: a #GtkToolbar.
+ * @widget: a #GtkWidget to add to the toolbar. 
+ * @tooltip_text: the element's tooltip.
+ * @tooltip_private_text: used for context-sensitive help about this toolbar element.
+ * @position: the number of widgets to insert this widget after.
+ * 
+ * Inserts a widget in the toolbar at the given position.
+ **/ 
 void
 gtk_toolbar_insert_widget (GtkToolbar *toolbar,
 			   GtkWidget  *widget,
@@ -1243,6 +1364,29 @@ gtk_toolbar_prepend_element (GtkToolbar          *toolbar,
 				     icon, callback, user_data, 0);
 }
 
+/**
+ * gtk_toolbar_insert_element:
+ * @toolbar: a #GtkToolbar.
+ * @type: a value of type #GtkToolbarChildType that determines what @widget
+ *   will be.
+ * @widget: a #GtkWidget, or %NULL. 
+ * @text: the element's label.
+ * @tooltip_text: the element's tooltip.
+ * @tooltip_private_text: used for context-sensitive help about this toolbar element.
+ * @icon: a #GtkWidget that provides pictorial representation of the element's function.
+ * @callback: the function to be executed when the button is pressed.
+ * @user_data: any data you wish to pass to the callback.
+ * @position: the number of widgets to insert this element after.
+ *
+ * Inserts a new element in the toolbar at the given position. 
+ *
+ * If @type == %GTK_TOOLBAR_CHILD_WIDGET, @widget is used as the new element.
+ * If @type == %GTK_TOOLBAR_CHILD_RADIOBUTTON, @widget is used to determine
+ * the radio group for the new element. In all other cases, @widget must
+ * be %NULL.
+ *
+ * Return value: the new toolbar element as a #GtkWidget.
+ **/
 GtkWidget *
 gtk_toolbar_insert_element (GtkToolbar          *toolbar,
 			    GtkToolbarChildType  type,
@@ -1266,7 +1410,7 @@ gtk_toolbar_insert_element (GtkToolbar          *toolbar,
   return gtk_toolbar_internal_insert_element (toolbar, type, widget, text,
                                               tooltip_text, tooltip_private_text,
                                               icon, callback, user_data,
-                                              position, FALSE);
+                                              position);
 }
 
 static GtkWidget *
@@ -1279,8 +1423,7 @@ gtk_toolbar_internal_insert_element (GtkToolbar          *toolbar,
                                      GtkWidget           *icon,
                                      GtkSignalFunc        callback,
                                      gpointer             user_data,
-                                     gint                 position,
-                                     gboolean             has_mnemonic)
+                                     gint                 position)
 {
   GtkToolbarChild *child;
   GtkWidget *box;
@@ -1341,8 +1484,8 @@ gtk_toolbar_internal_insert_element (GtkToolbar          *toolbar,
       GTK_WIDGET_UNSET_FLAGS (child->widget, GTK_CAN_FOCUS);
 
       if (callback)
-	gtk_signal_connect (GTK_OBJECT (child->widget), "clicked",
-			    callback, user_data);
+	g_signal_connect (child->widget, "clicked",
+			  callback, user_data);
 
       if (toolbar->style == GTK_TOOLBAR_BOTH_HORIZ)
 	  box = gtk_hbox_new (FALSE, 0);
@@ -1353,10 +1496,7 @@ gtk_toolbar_internal_insert_element (GtkToolbar          *toolbar,
 
       if (text)
 	{
-          if (has_mnemonic)
-            child->label = gtk_label_new_with_mnemonic (text);
-          else
-            child->label = gtk_label_new (text);
+	  child->label = gtk_label_new (text);
 	  gtk_box_pack_end (GTK_BOX (box), child->label, FALSE, FALSE, 0);
 	  if (toolbar->style != GTK_TOOLBAR_ICONS)
 	    gtk_widget_show (child->label);
@@ -1405,7 +1545,7 @@ gtk_toolbar_set_orientation (GtkToolbar     *toolbar,
 {
   g_return_if_fail (GTK_IS_TOOLBAR (toolbar));
 
-  gtk_signal_emit (GTK_OBJECT (toolbar), toolbar_signals[ORIENTATION_CHANGED], orientation);
+  g_signal_emit (toolbar, toolbar_signals[ORIENTATION_CHANGED], 0, orientation);
 }
 
 /**
@@ -1432,7 +1572,7 @@ gtk_toolbar_set_style (GtkToolbar      *toolbar,
   g_return_if_fail (GTK_IS_TOOLBAR (toolbar));
 
   toolbar->style_set = TRUE;
-  gtk_signal_emit (GTK_OBJECT (toolbar), toolbar_signals[STYLE_CHANGED], style);
+  g_signal_emit (toolbar, toolbar_signals[STYLE_CHANGED], 0, style);
 }
 
 /**
@@ -1447,7 +1587,7 @@ gtk_toolbar_set_style (GtkToolbar      *toolbar,
 GtkToolbarStyle
 gtk_toolbar_get_style (GtkToolbar *toolbar)
 {
-  g_return_val_if_fail (GTK_IS_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH);
+  g_return_val_if_fail (GTK_IS_TOOLBAR (toolbar), DEFAULT_TOOLBAR_STYLE);
 
   return toolbar->style;
 }
@@ -1468,13 +1608,17 @@ gtk_toolbar_unset_style (GtkToolbar *toolbar)
 
   if (toolbar->style_set)
     {
-      g_object_get (gtk_settings_get_default (),
-                    "gtk-toolbar-style",
-                    &style,
-                    NULL);
+      GtkSettings *settings = toolbar_get_settings (toolbar);
+
+      if (settings)
+	g_object_get (settings,
+		      "gtk-toolbar-style", &style,
+		      NULL);
+      else
+	style = DEFAULT_TOOLBAR_STYLE;
 
       if (style != toolbar->style)
-        gtk_signal_emit (GTK_OBJECT (toolbar), toolbar_signals[STYLE_CHANGED], style);
+        g_signal_emit (toolbar, toolbar_signals[STYLE_CHANGED], 0, style);
       
       toolbar->style_set = FALSE;
     }
@@ -1544,6 +1688,16 @@ gtk_real_toolbar_orientation_changed (GtkToolbar     *toolbar,
     }
 }
 
+static GtkWidget *
+get_first_child (GtkContainer *container)
+{
+  GList *children = gtk_container_get_children (container);
+  GtkWidget *result = children ? children->data : NULL;
+  g_list_free (children);
+  
+  return result;
+}
+
 static void
 gtk_real_toolbar_style_changed (GtkToolbar      *toolbar,
 				GtkToolbarStyle  style)
@@ -1592,19 +1746,19 @@ gtk_real_toolbar_style_changed (GtkToolbar      *toolbar,
 		if (child->label && !GTK_WIDGET_VISIBLE (child->label))
 		  gtk_widget_show (child->label);
 
-		box = (GtkWidget*)gtk_container_get_children (GTK_CONTAINER (child->widget))->data;
+		box = get_first_child (GTK_CONTAINER (child->widget));
 
 		if (GTK_IS_HBOX (box))
 		{
 		    if (child->icon)
 		    {
-			gtk_object_ref (GTK_OBJECT (child->icon));
+			g_object_ref (child->icon);
 			gtk_container_remove (GTK_CONTAINER (box),
 					      child->icon);
 		    }
 		    if (child->label)
 		    {
-			gtk_object_ref (GTK_OBJECT (child->label));
+			g_object_ref (child->label);
 			gtk_container_remove (GTK_CONTAINER (box),
 					      child->label);
 		    }
@@ -1617,12 +1771,12 @@ gtk_real_toolbar_style_changed (GtkToolbar      *toolbar,
 		    if (child->label)
 		    {
 			gtk_box_pack_end (GTK_BOX (box), child->label, FALSE, FALSE, 0);
-			gtk_object_unref (GTK_OBJECT (child->label));
+			g_object_unref (child->label);
 		    }
 		    if (child->icon)
 		    {
 			gtk_box_pack_end (GTK_BOX (box), child->icon, FALSE, FALSE, 0);
-			gtk_object_unref (GTK_OBJECT (child->icon));
+			g_object_unref (child->icon);
 		    }
 		    gtk_container_add (GTK_CONTAINER (child->widget),
 				       box);
@@ -1636,19 +1790,19 @@ gtk_real_toolbar_style_changed (GtkToolbar      *toolbar,
 		if (child->label && !GTK_WIDGET_VISIBLE (child->label))
 		  gtk_widget_show (child->label);
 
-		box = (GtkWidget*)gtk_container_get_children (GTK_CONTAINER (child->widget))->data;
+		box = get_first_child (GTK_CONTAINER (child->widget));
 		
 		if (GTK_IS_VBOX (box))
 		{
 		    if (child->icon)
 		    {
-			gtk_object_ref (GTK_OBJECT (child->icon));
+			g_object_ref (child->icon);
 			gtk_container_remove (GTK_CONTAINER (box),
 					      child->icon);
 		    }
 		    if (child->label)
 		    {
-			gtk_object_ref (GTK_OBJECT (child->label));
+			g_object_ref (child->label);
 			gtk_container_remove (GTK_CONTAINER (box),
 					      child->label);
 		    }
@@ -1661,12 +1815,12 @@ gtk_real_toolbar_style_changed (GtkToolbar      *toolbar,
 		    if (child->label)
 		    {
 			gtk_box_pack_end (GTK_BOX (box), child->label, TRUE, TRUE, 0);
-			gtk_object_unref (GTK_OBJECT (child->label));
+			g_object_unref (child->label);
 		    }
 		    if (child->icon)
 		    {
 			gtk_box_pack_end (GTK_BOX (box), child->icon, FALSE, FALSE, 0);
-			gtk_object_unref (GTK_OBJECT (child->icon));
+			g_object_unref (child->icon);
 		    }
 		    gtk_container_add (GTK_CONTAINER (child->widget), box);
 		    
@@ -1689,9 +1843,10 @@ static GtkReliefStyle
 get_button_relief (GtkToolbar *toolbar)
 {
   GtkReliefStyle button_relief = GTK_RELIEF_NORMAL;
-  
+
+  gtk_widget_ensure_style (GTK_WIDGET (toolbar));
   gtk_widget_style_get (GTK_WIDGET (toolbar),
-                        "button_relief", &button_relief,
+			"button_relief", &button_relief,
                         NULL);
 
   return button_relief;
